var path = require('path')
var webpack = require('webpack')
const VueLoaderPlugin = require('vue-loader/lib/plugin')
const CleanWebpackPlugin = require('clean-webpack-plugin')
const NODE_ENV = process.env.NODE_ENV
const isDev = NODE_ENV === 'development'
const isPro = NODE_ENV === 'production'

module.exports = {
    entry: {
        bpview: isDev ? './src/main.js' : './src/components/bpView.js'
    },
    // entry: NODE_ENV == 'development' ? './src/main.js' : './src/components/bpView.js',
    output: {
        path: path.resolve(__dirname, './dist'),
        publicPath: '/dist/',
        filename: '[name].js',
        library: 'bpview', // 引用时的模块名
        libraryTarget: 'umd', // 输出格式
        umdNamedDefine: true
    },
    module: {
        rules: [
            {
                test: /\.(c|le)ss$/,
                use: [
                    'style-loader',
                    'css-loader',
                    {
                        loader: 'postcss-loader',
                        options: {
                            plugins: [require('autoprefixer')]
                        }
                    },
                    'less-loader'
                ],
            }, 
            {
                test: /\.vue$/,
                loader: 'vue-loader',
                options: {}
            },
            {
                test: /\.js$/,
                loader: 'babel-loader',
                exclude: /node_modules/
            },
            {
                test: /\.(png|jpg|gif|svg)$/,
                loader: 'file-loader',
                options: {
                    name: '[name].[ext]?[hash]'
                }
            }
        ]
    },
    resolve: {
        alias: {
            'vue$': 'vue/dist/vue.esm.js',
            '@': path.resolve(__dirname, 'src'),
        },
        extensions: ['.js', '.vue', '.json']
    },
    devServer: {
        openPage: './index.html',
        noInfo: true,
        overlay: true
    },
    performance: {
        hints: false
    },
    devtool: '#eval-source-map',
    // optimization: {
    //     splitChunks: {
    //         cacheGroups: {
    //             vendors: {
    //                 name: 'vendors',
    //                 chunks: 'all',
    //                 test: /[\\/]node_modules[\\/]/,
    //                 priority: 100,
    //                 minChunks: 1,
    //                 minSize: 3000,
    //                 maxInitialRequests: 3,
    //                 maxAsyncRequests: 5,
    //             },
    //             common: {
    //                 name: 'common',
    //                 chunks: 'all',
    //                 test: /[\\/]src[\\/]*\.js/,
    //                 priority: 99,
    //                 minChunks: 1,
    //                 minSize: 0
    //             }
    //         }
    //     },
    //     runtimeChunk: {
    //         name: 'manifest'
    //     },
    //     minimizer: []
    // },
    plugins: [
        new VueLoaderPlugin()
    ]
}

if (process.env.NODE_ENV === 'production') {
    module.exports.devtool = '#source-map'
    // http://vue-loader.vuejs.org/en/workflow/production.html
    module.exports.plugins = (module.exports.plugins || []).concat([
        new CleanWebpackPlugin(['dist']),
        new webpack.DefinePlugin({
            'process.env': {
                NODE_ENV: '"production"'
            }
        }),
        new webpack.LoaderOptionsPlugin({
            minimize: true
        })
    ])
}